"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.showArmWarning = void 0;
const path = require("path");
const fs = require("fs/promises");
const parseArgs = require("yargs-parser");
const chalk = require("chalk");
const tempy = require("tempy");
const patch_apk_1 = require("./patch-apk");
const patch_app_bundle_1 = require("./patch-app-bundle");
const apktool_1 = require("./tools/apktool");
const uber_apk_signer_1 = require("./tools/uber-apk-signer");
const user_error_1 = require("./utils/user-error");
const { version } = require('../package.json');
async function main() {
    const args = parseArgs(process.argv.slice(2), {
        string: ['apktool', 'certificate', 'tmp-dir'],
        boolean: ['help', 'skip-patches', 'wait', 'debuggable', 'keep-tmp-dir'],
    });
    if (args.help) {
        showHelp();
        process.exit();
    }
    const [input] = args._;
    if (!input) {
        showHelp();
        process.exit(1);
    }
    const inputPath = path.resolve(process.cwd(), input);
    const fileExtension = path.extname(input);
    const baseName = path.basename(input, fileExtension);
    const outputName = `${baseName}-patched${fileExtension}`;
    const outputPath = path.resolve(path.dirname(inputPath), outputName);
    let isAppBundle = false;
    let taskFunction;
    switch (fileExtension) {
        case '.apk':
            taskFunction = patch_apk_1.default;
            break;
        case '.xapk':
            isAppBundle = true;
            taskFunction = patch_app_bundle_1.patchXapkBundle;
            break;
        case '.apks':
        case '.zip':
            isAppBundle = true;
            taskFunction = patch_app_bundle_1.patchApksBundle;
            break;
        default:
            showSupportedExtensions();
    }
    // Initialize and validate certificate path
    let certificatePath;
    if (args.certificate) {
        certificatePath = path.resolve(process.cwd(), args.certificate);
        let certificateExtension = path.extname(certificatePath);
        if (certificateExtension !== '.pem' && certificateExtension !== '.der')
            showSupportedCertificateExtensions();
    }
    let tmpDir = args['tmp-dir']
        ? path.resolve(process.cwd(), args['tmp-dir'])
        : tempy.directory({ prefix: 'apk-mitm-' });
    await fs.mkdir(tmpDir, { recursive: true });
    process.chdir(tmpDir);
    const apktool = new apktool_1.default({
        frameworkPath: path.join(tmpDir, 'framework'),
        customPath: args.apktool,
    });
    const uberApkSigner = new uber_apk_signer_1.default();
    showVersions({ apktool, uberApkSigner });
    console.log(chalk.dim(`  Using temporary directory:\n  ${tmpDir}\n`));
    taskFunction({
        inputPath,
        outputPath,
        certificatePath,
        tmpDir,
        apktool,
        uberApkSigner,
        wait: args.wait,
        skipPatches: args.skipPatches,
        isAppBundle,
        debuggable: args.debuggable,
    })
        .run()
        .then(async (context) => {
        if (taskFunction === patch_apk_1.default && context.usesAppBundle) {
            patch_apk_1.showAppBundleWarning();
        }
        console.log(chalk `\n  {green.inverse  Done! } Patched file: {bold ./${outputName}}\n`);
        if (!args['keep-tmp-dir']) {
            try {
                await fs.rm(tmpDir, { recursive: true, force: true });
            }
            catch (error) {
                // No idea why Windows gives us an `EBUSY: resource busy or locked`
                // error here, but deleting the temporary directory isn't the most
                // important thing in the world, so let's just ignore it
                const ignoreError = process.platform === 'win32' && error.code === 'EBUSY';
                if (!ignoreError)
                    throw error;
            }
        }
    })
        .catch((error) => {
        const message = getErrorMessage(error, { tmpDir });
        console.error([
            '',
            chalk `  {red.inverse.bold  Failed! } An error occurred:`,
            '',
            message,
            '',
            `  The full logs of all commands are available here:`,
            `  ${path.join(tmpDir, 'logs')}`,
            '',
        ].join('\n'));
        if (process.arch.startsWith('arm'))
            showArmWarning();
        process.exit(1);
    });
}
function getErrorMessage(error, { tmpDir }) {
    // User errors can be shown without a stack trace
    if (error instanceof user_error_1.default)
        return error.message;
    // Errors from commands can also be shown without a stack trace
    if (error.all)
        return formatCommandError(error.all, { tmpDir });
    return error.stack;
}
function formatCommandError(error, { tmpDir }) {
    return (error
        // Replace mentions of the (sometimes very long) temporary directory path
        .replace(new RegExp(tmpDir, 'g'), chalk `{bold <tmp_dir>}`)
        // Highlight (usually relevant) warning lines in Apktool output
        .replace(/^W: .+$/gm, line => chalk `{yellow ${line}}`)
        // De-emphasize Apktool info lines
        .replace(/^I: .+$/gm, line => chalk `{dim ${line}}`)
        // De-emphasize (not very helpful) Apktool "could not exec" error message
        .replace(/^.+brut\.common\.BrutException: could not exec.+$/gm, line => chalk `{dim ${line}}`));
}
function showHelp() {
    console.log(chalk `
  $ {bold apk-mitm} <path-to-apk/xapk/apks>

  {blue {dim.bold *} Optional flags:}
  {dim {bold --wait} Wait for manual changes before re-encoding}
  {dim {bold --tmp-dir <path>} Where temporary files will be stored}
  {dim {bold --keep-tmp-dir} Don't delete the temporary directory after patching}
  {dim {bold --debuggable} Make the patched app debuggable}
  {dim {bold --skip-patches} Don't apply any patches (for troubleshooting)}
  {dim {bold --apktool <path-to-jar>} Use custom version of Apktool}
  {dim {bold --certificate <path-to-pem/der>} Add specific certificate to network security config}
  `);
}
/**
 * Error that is shown when the file provided through the positional argument
 * has an unsupported extension. Exits with status 1 after showing the message.
 */
function showSupportedExtensions() {
    console.log(chalk `{yellow
  It looks like you tried running {bold apk-mitm} with an unsupported file type!

  Only the following file extensions are supported: {bold .apk}, {bold .xapk}, and {bold .apks} (or {bold .zip})
  }`);
    process.exit(1);
}
/**
 * Error that is shown when the file provided through the `--certificate` flag
 * has an unsupported extension. Exits with status 1 after showing the message.
 */
function showSupportedCertificateExtensions() {
    console.log(chalk `{yellow
  It looks like the certificate file you provided is unsupported!

  Only {bold .pem} and {bold .der} certificate files are supported.
  }`);
    process.exit(1);
}
function showVersions({ apktool, uberApkSigner, }) {
    console.log(chalk `
  {dim ╭} {blue {bold apk-mitm} v${version}}
  {dim ├ {bold apktool} ${apktool.version.name}
  ╰ {bold uber-apk-signer} ${uberApkSigner.version.name}}
  `);
}
function showArmWarning() {
    console.log(chalk `{yellow
  {inverse.bold  NOTE }

  {bold apk-mitm} doesn't officially support ARM-based devices (like Raspberry Pi's)
  at the moment, so the error above might be a result of that. Please try
  patching this APK on a device with a more common CPU architecture like x64
  before reporting an issue.
  }`);
}
exports.showArmWarning = showArmWarning;
main();
